 # 
 # Copyright (C) 2012-2019 Daniel Manrique-Vallier
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #

#####################
# Functions for accessing MCZ code.
# - Requires functions on ../GeneralCode/ArrayUtils.R

#SEXP R_Check_MCZ_Disjoint(SEXP MCZ_Matrix_COrder, SEXP levelsJ);
fn_R_MCZ_Check_Disjoint <- function(MZCmatrix){
  m <- fn_dataframe2num_matrix(MZCmatrix, offset=-1, missing=-1, C_style=T)
  lev <- fn_df_nlevels(MZCmatrix)
  return(  .Call('R_Check_MCZ_Disjoint', as.integer(m), as.integer(lev)) == 1);
}

#SEXP R_MCZ_Process(SEXP MCZ_Matrix_COrder, SEXP levelsJ);
fn_R_MCZ_Process <- function(MZCmatrix){
  if (fn_R_MCZ_Check_Disjoint(MZCmatrix)) return (MZCmatrix);
  m <- fn_dataframe2num_matrix(MZCmatrix, offset=-1, missing=-1, C_style=T)
  lev <- fn_df_nlevels(MZCmatrix)
  tmp <- .Call('R_MCZ_Process', as.integer(m), as.integer(lev))
  tmp <- matrix(tmp, byrow=T, ncol = NCOL(MZCmatrix))
  tmp <- fn_numeric_to_factor(tmp, list_levls=fn_df_levels(MZCmatrix), missing=-1, offset=-1)
  names(tmp) <- names(MZCmatrix)
  return(tmp)
}
fn_R_MCZ_nCells <- function(MCZ){
  M <- fn_R_MCZ_Process(MCZ)
  sum <- 0;
  for (i in 1:NROW(M)){
    sum <- sum + prod(fn_df_nlevels(d=M[i,which(is.na(M[i,]))]))
  }
  return(sum)
}

