/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <vector>
#include <iterator>
#include "definitions.h"
#include "margin_conditions.h"
#include "CData_DM_MCZ.h"


//--------------------------------------------------------------------------------
// Implementation of class CData_DM_MCZ
//--------------------------------------------------------------------------------

//These are functions to deal with the 
// margin condition processing and testing.

void CData_DM_MCZ::Process_MC(){
	// takes ZeroMC_IJ and converts it into an exclusive list
	//If it is already exclusive, just copies it.

	std::list<std::vector<int> > newzeros;
	if (this->nZeroMC_original == 0){
		this->nZeroMC = 0;
	} else {
		MC_to_MCpartitionList(ZeroMC_IJ_original, J, nZeroMC_original, levelsJ, newzeros);
		this->nZeroMC = newzeros.size();
	}
	//store the result.
	_Declare_and_Allocate_derived("ZeroMC_IJ", CPar_Data_Type::T_INT, 2, nZeroMC, J);
	_set_internal_pointer("ZeroMC_IJ", &ZeroMC_IJ);
	//load the data
	int i = 0;
	for (std::list<std::vector<int> >::iterator it = newzeros.begin(); it != newzeros.end(); it++){
		std::copy(it->begin(), it->end(), ZeroMC_IJ[i++]);
	}
}

bool CData_DM_MCZ::Validate_Data(){
	//test that all responses are not in the zeros list
	for (int i = 0; i < n; i++){
		for (int c = 0; c < nZeroMC_original; c++){
			if(!check_disjoint(x[i], x[i]+J, this->ZeroMC_IJ_original[c])){
				return(false);
			}
		}
	}
	return(true);
}

void CData_DM_MCZ::MCZ_Coordinates(std::vector<int> &answer){
	std::set<int> resp;
	answer.clear();
	for (int i = 0; i < this->nZeroMC; i++){
		for (int j = 0; j < J; j++){
			if (this->ZeroMC_IJ[i][j] != -1) resp.insert(j);
		}
	}
	std::copy(resp.begin(), resp.end(), std::back_inserter(answer));
	std::sort(answer.begin(), answer.end());
}



//Get the list of *detectable* errors (indexing starts at 1)
std::vector<int> CData_DM_MCZ::get_inconsistent_rows(){
	std::vector<int> v;
	for (int i = 0; i < n; i++){
		for (int c = 0; c < nZeroMC; c++){
			if(!check_x_notin_mu(x[i], x[i] + J, this->ZeroMC_IJ[c])){
				v.push_back(i + 1);
				break;
			}
		}
	}
	return v;
}
