/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

//2016-28-2016: Modified to accept no MCZs. IN that case just set nZeros = 0 and the pointer to the 
// data source = NULL.
#ifndef _CData_DM_MCZ_H
#define _CData_DM_MCZ_H

#include "CData_DM.h"
#include "margin_conditions.h"

#include <set>
#include <vector>

class CData_DM_MCZ : public CData_DM{
public:
	CData_DM_MCZ() : CData_DM(){
		Declare();
	}
	int** ZeroMC_IJ; //processed structural zero definitions.
	int** ZeroMC_IJ_original; //original structural zero definitions.
	int nZeroMC;
	int nZeroMC_original;
	void Process_MC();
	void Set_Manually(int *x_flat, int _J, int _n, int* _ZeroMC_flat, int _nZeroMC, int *_levels) {
		CData_DM::Set_Manually(x_flat, _J, _n, _levels);
		nZeroMC_original = _nZeroMC;
		_Load_Variable("ZeroMC_IJ_original", _ZeroMC_flat, _nZeroMC, _J);
	}	
	bool Validate_Data();
	void MCZ_Coordinates(std::vector<int> &answer);
	std::vector<int> get_inconsistent_rows();
protected:
	void Close_Loading(){
		CData_DM::Close_Loading();
		nZeroMC_original = data_container["ZeroMC_IJ_original"].get_dim_lengths()[0]; //redundant.
		_set_internal_pointer("ZeroMC_IJ_original", &(CData_DM_MCZ::ZeroMC_IJ_original));
		//_Declare_Variable("ZeroMC_IJ", CPar_Data_Type::T_INT, 2, true); // Declare here because this is not the original read data.
		Process_MC(); //this function will create and fill a CVariable_Container "ZeroMC_IJ"
		//_set_internal_pointer("ZeroMC_IJ", &ZeroMC_IJ); //already done in Process_MC()
		//_set_internal_pointer("ZeroMC_IJ_original", &ZeroMC_IJ_original); //already done it.
	}
private:
	void Declare(){
		_Declare_Variable("ZeroMC_IJ_original", CPar_Data_Type::T_INT, 2, false);
	}
};
#endif  //_CData_DM_MCZ_H
