/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef _CLcm_Bay_H
#define _CLcm_Bay_H

#include <stdio.h>
#include <stdlib.h>
#include "daniel2/CChain.h"
#include "daniel2/CParam.h"
#include "daniel2/dan_sys.h"
#include "CData_DM.h"

class CParams_Lcm_Bay : public CParam {
public:
	//parameters

	int *zI; //(z_i = k) i=1..N_MAX //just n
	double ***psiJKL; //(psi_{jkl}) l -> # of levels
	double *nuK;
	int *count0K;
	int *count1K;
	//priors
	//structure
	int J, K, L, *levelsJ, n;
	//auxiliary
	int*** aux_dirJKL;

	//Constructors.
	CParams_Lcm_Bay(int J, int K, int *levelsJ, int n){
		class_construct(J, K, levelsJ, n);
	}
	CParams_Lcm_Bay(CData_DM* dat, int K){
		class_construct(dat->J,  K, dat->levelsJ, dat->n);		
	}
	CParams_Lcm_Bay(CParams_Lcm_Bay& orig){
		class_construct(orig.J,  orig.K, orig.levelsJ, orig.n);
		//*this = orig;
		storage = orig.storage;
	}
	//~CParams_Lcm_Bay(); //Destructor
	void initizalize(gsl_rng *r); //Initialize the parameters
	void readInits();
	void openFiles();
	virtual void predict(int* xIJ_flat, double *ret, int I){
		int * restrict cellJ = xIJ_flat;
		double * restrict p_sum = ret;
		for (int i = 0; i < I; ++i, cellJ += J, ++p_sum){
			*p_sum = 0.0;
			for(int k = 0; k < K; ++k){
				double prod = nuK[k];
				for(int j = 0; j < J; ++j){
					int l = cellJ[j];
					if ( l != -1 ) prod *= psiJKL[j][k][l];
				}
				*p_sum += prod;
			}
		}
	}

	//no need to override this one as long as the overriding of predict(...) is ok.
	double predict (int* cellJ){
		double ret;
		predict(cellJ, &ret, 1);
		return(ret);
	}
private:
	void class_construct(int J, int K, int *levelsJ, int n);
};
/////////////////////////////////
/////////////////////////////////

class CLcm_Bay : public CChain {
public:
	CLcm_Bay(CData_DM *_data, CParams_Lcm_Bay *_par) : CChain(_par), par(_par), data(_data){
		class_construct();
	}
	void Update();
	CParams_Lcm_Bay *par; 
	double predict(int* xJ);
private:
	CData_DM* data;
	//int ***aux_dirJKL;
	void class_construct(){
		//aux_dirJKL = par->aux_dirJKL;
		setModelSignature("CLcm_Bay");
	}
protected:
	//int J, n; //to be set with values from *data.
	//int K; //to be set with values from *par or in constructor.
	//Full conditional samplers
	//  all these functions update the values on the *par object.
	void sam_nu();
	void sam_psi();
	void sam_z();
};


#endif  //_CLcm_Bay_CR_H
