/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef _CLcm_Bay_Reg_H
#define _CLcm_Bay_Reg_H

#include <stdio.h>
#include <stdlib.h>
#include "daniel2/CChain.h"
#include "daniel2/CParam.h"
#include "CData_DM.h"
#include "CLcm_Bay.h"

class CParams_Lcm_Bay_Reg : public CParams_Lcm_Bay {
public:
	//parameters
	double *log_nuK;
	double alpha;
	int k_star;
	//priors
	double a_alpha, b_alpha; // alpha ~ Gamma[a_alpha, b_alpha]

	//functions
	CParams_Lcm_Bay_Reg(int J, int K, int *levelsJ, int n, double a_alpha, double b_alpha)
		:	CParams_Lcm_Bay(J, K, levelsJ, n), a_alpha(a_alpha), b_alpha(b_alpha){
		class_construct();
	}
	CParams_Lcm_Bay_Reg(CData_DM* dat, int K, double a_alpha, double b_alpha, char* file_dir_out, char* file_out_prefix)
		:	CParams_Lcm_Bay(dat, K), a_alpha(a_alpha), b_alpha(b_alpha){
		class_construct();
	}		
	CParams_Lcm_Bay_Reg(CParams_Lcm_Bay_Reg& orig) : CParams_Lcm_Bay(orig), a_alpha(orig.a_alpha), b_alpha(orig.b_alpha) {
		class_construct();
		this->storage = orig.storage;
	}
	void initizalize(gsl_rng *r); //Initialize the parameters
	void compute_k_star(){
		static std::vector<int> Kcomb(K);
		for (int k = 0; k < K; k++){
			Kcomb[k] = (count0K[k] > 0) || (count1K[k] > 0);
		}
		k_star = K - std::count(Kcomb.begin(), Kcomb.end(), 0);
	}
private:
	void class_construct(){
		//assumes that the parent class has been properly intialized.
		_add_parameter("log_nuK", CPar_Data_Type::T_DOUBLE, &log_nuK, 1, K);
		_add_existing_scalar("k_star", CPar_Data_Type::T_INT, &k_star);
		_add_existing_scalar("a_alpha", CPar_Data_Type::T_DOUBLE, &a_alpha);
		_add_existing_scalar("b_alpha", CPar_Data_Type::T_DOUBLE, &b_alpha);
		_add_existing_scalar("alpha", CPar_Data_Type::T_DOUBLE, &alpha);
	}
};
/////////////////////////////////
/////////////////////////////////

class CLcm_Bay_Reg : public CChain{
public:
	CLcm_Bay_Reg(){}; //default constructor. DO NOT USE!!!!
	CLcm_Bay_Reg(CData_DM *_data, CParams_Lcm_Bay_Reg *_par) : CChain(_par), par(_par) , data(_data) {
		class_construct();
	}
	void Update();
	CParams_Lcm_Bay_Reg *par; 
	void WriteInfoSheet(char *filename);
	double predict(int* xJ);
private:
	CData_DM* data;
	//int ***aux_dirJKL;
	void class_construct(){
		//aux_dirJKL = par->aux_dirJKL;
		setModelSignature("CLcm_Bay_Reg");
	}
protected:
	//Full conditional samplers
	//  all these functions update the values on the *par object.
	void sam_nu();
	void sam_psi();
	void sam_z();
	void sam_alpha();
};


#endif  //_CLcm_Bay_Reg_H
