/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef _CLcm_EdImp_H
#define _CLcm_EdImp_H

#include <vector>
#include "daniel2/CChain.h"
#include "daniel2/CParam.h"
#include "CData_DM_MCZ.h"
#include "CLcm_Bay_Reg.h"
class CParams_Lcm_EdImp : public CParams_Lcm_Bay_Reg {
public:
	int **xIJ; //for holding data and augmented data.
	int **x_synthIJ; //for purelly synthetic data.
	int **eIJ; //Matrix of errors 1=error; 0=ok.
	int **eMaskIJ;// 0 = known to be 0; -1 = not known. We're not considering "known to be in error" because it complicates things too much. 
 	//int *e_marginal_J; //marginal counts for eIJ
	//double *epsilonJ; //probabilities of error for each component of the data vector
 	int e_marginal; //marginal counts for eIJ
	double epsilon; //probabilities of error for each component of the data vector
	double *qJ; //uniform probabilities of another value given error.
	double a_error, b_error; //hyperparameters
	int **MCZ; //for holing a local copy of marginal conditions. 
	double *pZeroMC_I, prob_zero;
	int *z2_Nmax;
	int **x2_NMax_J;
	unsigned int *count_partition; //imputed number of individuals in each partition
	int* permutationJ; //order of sampling X's
	int Nmis, N_mis_max, nZeroMC;
	double *tmp_doubleJ; //temporary storage.
	int *tmp_intJ;
	CParams_Lcm_EdImp(int J, int K, int *levelsJ, int n, int Nmis_max, int **MCZ, int nZeroMC, double a_alpha, double b_alpha, double a_error, double b_error, int** x, int** _emask)
			:	CParams_Lcm_Bay_Reg(J, K, levelsJ, n, a_alpha, b_alpha), a_error(a_error), b_error(b_error){
				class_construct(Nmis_max, MCZ, nZeroMC, x, _emask);
	}
	CParams_Lcm_EdImp(CData_DM_MCZ* dat, int K, int Nmis_max,  double a_alpha, double b_alpha, double a_error, double b_error, int** _emask)
			:	CParams_Lcm_Bay_Reg(dat->J, K, dat->levelsJ, dat->n,  a_alpha, b_alpha), a_error(a_error), b_error(b_error){ 
				class_construct(Nmis_max, dat->ZeroMC_IJ, dat->nZeroMC, dat->x, _emask);
	}
	CParams_Lcm_EdImp(CParams_Lcm_EdImp &orig)
		:	CParams_Lcm_Bay_Reg(orig.J, orig.K, orig.levelsJ, orig.n,  orig.a_alpha, orig.b_alpha), a_error(orig.a_error), b_error(orig.b_error){ 
			class_construct(orig.N_mis_max, orig.MCZ, orig.nZeroMC, orig.xIJ, orig.eMaskIJ);
		//*this = orig;
		storage = orig.storage;
	}
	void initizalize(gsl_rng *r);
	void predict (int* xIJ_flat, double* res, int I){
		//renormalize base class prediction.
		CParams_Lcm_Bay::predict(xIJ_flat, res, I);
		const double renorm = 1.0 / (1.0 - prob_zero);
		for (double* it = res; it != res + I; ++it){
			*it *= renorm;
		}
	}
	void resize_Nmis_max(int newsize, bool nocopy = true){
		//Have to mantain: Nmis, N_mis_max, x2_NMax_J, z2_Nmax
		if (0.8 * newsize < Nmis){
			throw std::runtime_error("Error resizing Nmis_max. Current Nmis parameter larger than 80% of new size");
		}
		if (!nocopy){
			CVariable_Container z2tmp((*this)["z2_Nmax"]);
			CVariable_Container x2_tmp((*this)["x2_NMax_J"]);
			_remove_parameter("z2_Nmax");
			_remove_parameter("x2_NMax_J");
			_add_parameter("z2_Nmax", CPar_Data_Type::T_INT, &z2_Nmax, 1, newsize);
			_add_parameter("x2_NMax_J", CPar_Data_Type::T_INT, &x2_NMax_J, 2, newsize, J);
			std::copy((int*)(z2tmp.get_data_base()), (int*)(z2tmp.get_data_base()) + Nmis, z2_Nmax);
			std::copy((int*)(x2_tmp.get_data_base()), (int*)(x2_tmp.get_data_base()) + Nmis * J, x2_NMax_J[0]);
		} else {
			_remove_parameter("z2_Nmax");
			_remove_parameter("x2_NMax_J");
			_add_parameter("z2_Nmax", CPar_Data_Type::T_INT, &z2_Nmax, 1, newsize);
			_add_parameter("x2_NMax_J", CPar_Data_Type::T_INT, &x2_NMax_J, 2, newsize, J);
		}
		N_mis_max = newsize;
	}
	bool ValidateMask(int **data_xIJ);
	bool is_mask_unknown(const int &i);
	//CPar_Data_Type data_type;	
private:
	void class_construct(int Nmis_max, int** MCZ_, int nZeroMC, int **x, int **_emask);
	int dubious_coordinates(int* xiJ, int* answerJ);//Determines the coordinates of the values that make x belong to the exclusion set.
};

class CLcm_EdImp : public CChain{
public:
	//Constructors
	CLcm_EdImp(CData_DM_MCZ *_data, CParams_Lcm_EdImp *_par) : CChain(_par), par(_par), data(_data){
		class_construct();
	}

	void Update();
	CParams_Lcm_EdImp *par; 
	double predict(int* xJ);
	double predict_renorm(int* xJ);
	void Initializes();
private:
	CData_DM_MCZ* data;
	//int ***aux_dirJKL;
	void class_construct(){
		//aux_dirJKL = par->aux_dirJKL;
		setModelSignature("CLcm_EdImp");
	}
protected:
	//Full conditional samplers
	void sam_e_x(); //imputation step.
	void samp_x_i(const int& i);//imputation of a single row.
	void sam_e_x_Rejection(); //imputation step.
	void sam_e_x_Gibbs(); //imputation step.

	void sam_epsilon();
	void sam_nu(); 
	void sam_psi();
	void sam_z();
	void sam_Z2_X2();
	void sam_Nmis();
	void sam_alpha();
	void compute_probs_miss();
private:
	double _aj(const int& j, const int& k, const int& xj, const int* y);
	double _Conditional_X_j(const int& zi, const int& j, const std::vector<int>& xi);
	double _probs[200]; 
	int _A[200], _A_minus_Y[200];
};


#endif  //_CLcm_EdImp_CR_H
