/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef _CLCM_IMPUTATION_H
#define _CLCM_IMPUTATION_H

#include <vector>
#include "daniel2/CChain.h"
#include "daniel2/CParam.h"
#include "CData_DM_MCZ.h"
#include "CLcm_Bay_Reg.h"
class CParams_Lcm_Imputation : public CParams_Lcm_Bay_Reg {
public:
	int **xIJ; //for holding data and augmented data.
	int **MCZ; //for holing a local copy of marginal conditions. 
	double *pZeroMC_I, prob_zero;
	int *z2_Nmax;
	int **x2_NMax_J;
	unsigned int *count_partition; //imputed number of individuals in each partition
	int* permutationJ; //order of sampling X's
	int Nmis, N_mis_max, nZeroMC;
	double *tmp_doubleJ; //temporary storage.
	int *tmp_intJ;
	CParams_Lcm_Imputation(int J, int K, int *levelsJ, int n, int Nmis_max, int **MCZ, int nZeroMC, double a_alpha, double b_alpha, int** x)
			:	CParams_Lcm_Bay_Reg(J, K, levelsJ, n, a_alpha, b_alpha){
				class_construct(Nmis_max, MCZ, nZeroMC, x);
	}
	CParams_Lcm_Imputation(CData_DM_MCZ* dat, int K, int Nmis_max,  double a_alpha, double b_alpha)
			:	CParams_Lcm_Bay_Reg(dat->J, K, dat->levelsJ, dat->n,  a_alpha, b_alpha) { 
				class_construct(Nmis_max, dat->ZeroMC_IJ, dat->nZeroMC, dat->x);
	}
	CParams_Lcm_Imputation(CParams_Lcm_Imputation &orig)
		:	CParams_Lcm_Bay_Reg(orig.J, orig.K, orig.levelsJ, orig.n,  orig.a_alpha, orig.b_alpha){ 
			class_construct(orig.N_mis_max, orig.MCZ, orig.nZeroMC, orig.xIJ);
		//*this = orig;
		storage = orig.storage;
	}
	void initizalize(gsl_rng *r);
	void predict (int* xIJ_flat, double* res, int I){
		//renormalize base class prediction.
		CParams_Lcm_Bay::predict(xIJ_flat, res, I);
		const double renorm = 1.0 / (1.0 - prob_zero);
		for (double* it = res; it != res + I; ++it){
			*it *= renorm;
		}
	}

private:
	void class_construct(int Nmis_max, int** MCZ_, int nZeroMC, int **x);
};



class CLcm_Imputation : public CChain{
public:
	//Constructors
	CLcm_Imputation(CData_DM_MCZ *_data, CParams_Lcm_Imputation *_par) : CChain(_par), par(_par), data(_data){
		class_construct();
	}

	void Update();
	CParams_Lcm_Imputation *par; 
	double predict(int* xJ);
	double predict_renorm(int* xJ);
	void Initializes();
private:
	CData_DM_MCZ* data;
	//int ***aux_dirJKL;
	void class_construct(){
		//aux_dirJKL = par->aux_dirJKL;
		setModelSignature("CLcm_Imputation");
	}
protected:
	//Full conditional samplers
	void sam_x();
	void sam_nu(); 
	void sam_psi();
	void sam_z();
	void sam_Z2_X2();
	void sam_Nmis();
	void sam_alpha();
	void compute_probs_miss();
private:
	double _probs[200]; 
	int _A[200], _A_minus_Y[200];
};


#endif  //_CLcm_Imputation_CR_H
