/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <string>
#include <vector>
#include <set>
#include <algorithm>
#include "daniel2/CParams_generic.h"
#include "daniel2/Model_Environ.h"
#include "daniel2/R_Environ_Simple.h"
#include "daniel2/CVariable_Container.h"
#include "CData_DM_MCZ.h"
#include "CLcm_Bay_Reg_MCZ.h"
#include "margin_conditions.h"
#include "R_Lcm_EdImp.h"
//using namespace std;


//type defs with instantiations.

//Functions for creating the model environments.
void create_mask(const int& mask_type, CData_DM_MCZ& d, CVariable_Container& dst){
	//int* emask_raw = (int*) dst.get_data_base();
	int** mask = (int**) dst.get_data();
	switch (mask_type){
		case 0:
			dst.fill<int>(0);//by default "No error"
			break;
		case 1:
			dst.fill<int>(-1);//by default "Maybe error"
			break;
		case 2:
			dst.fill<int>(0);
			for (int i = 0; i < d.n; i++){
				bool in_MCZ = false;
				for (int ii = 0; ii < d.nZeroMC; ii++){
					if (!check_x_notin_mu(d.x[i], d.x[i] + d.J, d.ZeroMC_IJ[ii])){
						in_MCZ = true;
						break;
					}
				}
				if (in_MCZ){
					//borrar++;
					std::fill(mask[i], mask[i]+d.J, -1); //don't know if error.
				}
			}
			break;
	}
}

SEXP R_Create_Lcm_EdImp2(
		//data = list(levelsJ=numeric(J), x = array(x, dims=(n, J)), ZeroMC_IJ_original=array(dims=c(number of zeros,J))) 
		//SEXP data_list, 
    SEXP J, SEXP Idata, SEXP Izeros, SEXP x, SEXP ZeroMC_IJ_original, SEXP levelsJ, 
    SEXP K, SEXP Nmis_max, SEXP a_alpha, SEXP b_alpha, SEXP a_error, SEXP b_error, 
		SEXP mask_type,	SEXP len_buffer, SEXP subsamp){
	//mask_type: 0: No error; 1: all of them; 2: Only inconsistent.
	D* d;
	try{
		d = new CData_DM_MCZ();
	} catch (const std::exception& e){
		Rf_error("Cannot create data object: %s", e.what());
	}
//	try{
//		R_Load_CData(*d, data_list); //shoudl do everything automatically!
//	} catch (const std::exception& e){
//		Rf_error("Cannot load data: %s", e.what());
//	}

// [ ] <- HERE
//LOAD DATA INTO THE DATA OBJECT
//	void Set_Manually(int *x_flat, int _J, int _n, int* _ZeroMC_flat, int _nZeroMC, int *_levels) {
    //SEXP J, SEXP Idata, SEXP Izeros, SEXP x, SEXP ZeroMC_IJ_original, SEXP levelsJ, 
  d->Set_Manually(INTEGER(x), *INTEGER(J), *INTEGER(Idata), INTEGER(ZeroMC_IJ_original), *INTEGER(Izeros), INTEGER(levelsJ));
	//Create error mask. TO DO: Create options for different error masks.
	CVariable_Container mask_obj(CPar_Data_Type::T_INT, 2,"mask");
	mask_obj.allocate_space(2, d->n, d->J);
	create_mask(*INTEGER(mask_type), *d, mask_obj);
	P* par;
	Lcm_EdImp_Environ* e;
	try{
		par = new P(d, *INTEGER(K), *INTEGER(Nmis_max), *REAL(a_alpha), *REAL(b_alpha), *REAL(a_error), *REAL(b_error), (int**)mask_obj.get_data());
	} catch (const std::exception& e){
		Rf_error("Cannot create model object: %s", e.what());
	}
	try {
		e = new Lcm_EdImp_Environ(d, par, *INTEGER(len_buffer), *INTEGER(subsamp), true);
	} catch (const std::exception& e){
		Rf_error("Cannot create model object: %s", e.what());
	}

	SEXP ext = PROTECT(R_MakeExternalPtr((void*)e, R_NilValue, R_NilValue));
    R_RegisterCFinalizerEx(ext, finalizer_Env, TRUE);
    UNPROTECT(1);
	return(ext);
}

SEXP R_get_imputed_x_Lcm_EdImp(SEXP p){
   SEXP x = R_NilValue; 
   Lcm_EdImp_Environ* m = (Lcm_EdImp_Environ*)get_env(p);
   int n = m->get_raw_dataset_length();
    PROTECT(x = allocVector(INTSXP, n));
    m->get_current_imputed_x(INTEGER(x));
    UNPROTECT(1);
    return(x);
}

SEXP R_get_detectable_errors(SEXP p){
	SEXP ret = R_NilValue;
	Lcm_EdImp_Environ* m = (Lcm_EdImp_Environ*)get_env(p);
	const std::vector<int>& v = m->get_data().get_inconsistent_rows();
	const int* pv = &v.front();
	PROTECT( ret = allocVector(INTSXP, v.size()) );
	//memcpy(INTEGER(ret), pv, sizeof(int) * v.size());
	std::copy(pv, pv + v.size(), INTEGER(ret));
	UNPROTECT(1);
	return (ret);
}
