/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <string>
#include <vector>
#include <set>
#include <algorithm>
#include "daniel2/CParams_generic.h"
#include "daniel2/Model_Environ.h"
#include "daniel2/R_Environ_Simple.h"
#include "daniel2/CVariable_Container.h"
#include "CData_DM_MCZ.h"
#include "CLcm_Imputation.h"
#include "margin_conditions.h"
#include "R_Lcm_Imputation.h"
//using namespace std;


//Classes for this sampler.
typedef CData_DM_MCZ D;
typedef CLcm_Imputation M;
typedef CParams_Lcm_Imputation P;

//template definition
class Lcm_Imput_Environ : public CModel_Environ_Simple<D, M, P>{
public:
	typedef CModel_Environ_Simple<D, M, P> mit_t;
	Lcm_Imput_Environ(D* data, P* par, int len_buffer, int subsamp=1, bool del_objects=true) 
			:	mit_t(data, par, len_buffer, subsamp, del_objects){
	}
	~Lcm_Imput_Environ(){
	}
	void get_current_imputed_x(int *x_raw_imputed){
	    //copies the current imputed dataset (for generating multiply imputed datasets)
		//This is a way of not having to include x in the tracing thing.
		//This is to optimize memory usage.
		std::copy(this->par->xIJ[0], this->par->xIJ[0] + this->par->n * this->par->J, x_raw_imputed);
	}
protected:
	void Other_Tracing_Options(){
	}
	void Other_Updating_Options(){
		R_CheckUserInterrupt();
		//DAN_PRINTF("\riteration = %d       ", this->iteration + 1);
	}
};

//Functions for creating the model environments.
SEXP R_Create_Lcm_Imput(SEXP x_flat, SEXP J, SEXP n, SEXP ZeroMC_flat, SEXP nZeroMC, SEXP levels,
						  SEXP K, SEXP Nmis_max, SEXP a_alpha, SEXP b_alpha,
						  SEXP len_buffer, SEXP subsamp){
	//objects creation inside "try" block. This allows stack unwinding.
		Lcm_Imput_Environ* e;
		try {
			D* d = new D();
			d->Set_Manually(INTEGER(x_flat), *INTEGER(J), *INTEGER(n), INTEGER(ZeroMC_flat), *INTEGER(nZeroMC), INTEGER(levels));
			e = new Lcm_Imput_Environ(d,
				new P(d, *INTEGER(K), *INTEGER(Nmis_max),*REAL(a_alpha), *REAL(b_alpha)), 
				*INTEGER(len_buffer), *INTEGER(subsamp), true);	
		} catch (const std::exception& e){
			DAN_ERR_EXIT(e.what());
		}
	SEXP ext = PROTECT(R_MakeExternalPtr((void*)e, R_NilValue, R_NilValue));
    R_RegisterCFinalizerEx(ext, finalizer_Env, TRUE);
    UNPROTECT(1);
	return(ext);
}

SEXP R_get_imputed_x_Lcm_Imput(SEXP p){
   SEXP x = R_NilValue; 
   Lcm_Imput_Environ* m = (Lcm_Imput_Environ*)get_env(p);
   //sample!
   int size = m->get_data().J * m->get_data().n;
    PROTECT(x = allocVector(INTSXP, size));
    m->get_current_imputed_x(INTEGER(x));
    UNPROTECT(1);
    return(x);
}
