/* 
 * Copyright (C) 2007-2019 Daniel Manrique-Vallier
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "daniel2/dan_array_utils.h"
#include "margin_conditions.h"
#include "R_Margin_Conditions.h"
#include "daniel2/R_Environ_Simple.h"

SEXP R_MCZ_Process(SEXP MCZ_Matrix_COrder, SEXP levelsJ){
	int J = length(levelsJ);
	int I = length(MCZ_Matrix_COrder) / J;
	int** ZeroMC_IJ = (int **) dan_flat2array2D(INTEGER(MCZ_Matrix_COrder), sizeof(int), I, J);
	int size;
	int** newzeros = MC_to_MCpartition(ZeroMC_IJ, J, I, INTEGER(levelsJ), size);
	SEXP ret;
	PROTECT(ret = allocVector(INTSXP, size * J));
	std::copy(newzeros[0], newzeros[0]+ J * size, INTEGER(ret));
	dan_free_arrayND_overhead(ZeroMC_IJ);
	dan_free(newzeros[0]);
	dan_free_arrayND_overhead(newzeros);
	UNPROTECT(1);
	return(ret);
}

SEXP R_Check_MCZ_Disjoint(SEXP MCZ_Matrix_COrder, SEXP levelsJ){
	int J = length(levelsJ);
	int I = length(MCZ_Matrix_COrder) / J;
	int** ZeroMC_IJ = (int **) dan_flat2array2D(INTEGER(MCZ_Matrix_COrder), sizeof(int), I, J);
	//int size;
	//Returning an integer. I can't find how to return a simple boolean.
	SEXP ret;
	PROTECT(ret = allocVector(INTSXP, 1));
	*INTEGER(ret) = check_disjoint_MC(ZeroMC_IJ, I, J) ? 1 : 0 ;
	dan_free_arrayND_overhead(ZeroMC_IJ); 
	UNPROTECT(1);
	return(ret);
}

