###################################################################
# Edit imputation demo using EIDPM models
# See instructions in README file.
# (c) 2016 Daniel Manrique-Vallier
###################################################################


#Set the working directory to this file's location.
#setwd("C:/Data/Daniel4/Projects/LCM_EdImp/code_submission/demo")

#Read variables for this problem.
require(yaml)
vars <- yaml.load_file('variables.yml')

require(package = vars$pack_name, lib.loc = vars$pack_lib, character.only = T)
source('functions_for_tests.R')

#Sampling n=1000 individuals from the general population
load(paste(vars$data_location,'/','NY_2000_5pct.RData', sep = ''))
set.seed(1)
n <- 1000
indx <- sample(1:NROW(dat_proc), size = n, replace = T); samp <- dat_proc[indx,]

#contaminate the sample with 40\% error rate.
dat_cont <- fn_contaminate2(orig = samp, err_rate = rep(0.4, 10))

###
# Edit imputation

#A)EIDPM with e~beta(1,1) prior for the common error rate.
 #1) Create and initialize the sampler
eidpm <- lcm_EdImp_generator$new(x=dat_cont$result,ZeroMC=MCZ, K=25, Nmis_max = NA, a_alpha=0.25, b_alpha=0.25, 
                                      a_error = 1, b_error = 1,
                                      mask_type = 2,
                                      len_buffer=10000, subsamp=10)
eidpm$Init_Model()

# Generate 50 imputations, spaced every 100 samples, after 20,000 burn-in samples.
imputations <- edit_impute(eidpm, burnin = 20000, samples = 50, thining = 100)

# B)Fellegi Holt using NPLCM with structural zeros
#  using the package editrules from Edwin de Jonge and Mark van der Loo
require(editrules)
E <- fn_Create_Edit_Rules(MCZ)
el <- localizeErrors(E, dat_cont$result)
FH_imp <- lcmEdImp:::lcm_Imput_Generator$new(x=fn_blank_mask(dataset=dat_cont$result, mask=el$adapt),Nmis_max = 50000, ZeroMC=MCZ, K=25, a_alpha=0.25, b_alpha=0.25,  len_buffer=10000, subsamp=10)
#FH_imp <- lcm_Imput_Generator$new(x=fn_blank_mask(dataset=dat_cont$result, mask=el$adapt),Nmis_max = 50000, ZeroMC=MCZ, K=25, a_alpha=0.25, b_alpha=0.25,  len_buffer=10000, subsamp=10)
FH_imp$Init_Model()
FH_imp$Update(20000)
FH_imputations <- list()
for (i in 1:50){
  FH_imp$Update(100)
  FH_imputations[[i]] <- FH_imp$Get_Imputed_Dataset()
}

# Evaluation: Estimate all the 3-way marginal proportions. 
# 1) actual population values. 
actual <- sam_stat(dat_proc, m_way=3, N=n)$point

# 2) EIDPM imputed estimates. Just the means across the multiply imputed datasets.
s <- imputations[[1]]
eidpm_est <- numeric(length(sam_stat(s, m_way = 3, N = n)$point))

for (s in imputations){
  eidpm_est <- eidpm_est + sam_stat(s, m_way = 3, N = n)$point
}
eidpm_est <- eidpm_est/length(imputations)

# 3) FH imputed. Just the means across the multiply imputed datasets.
fh_est  <- numeric(length(sam_stat(s, m_way = 3, N = n)$point))
for (s in FH_imputations){
  fh_est <- fh_est + sam_stat(s, m_way = 3, N = n)$point
}
fh_est <- fh_est/length(FH_imputations)

#####
# Summaries. Comparing the estimated values with the population values

x11(10,5); par(mfrow=c(1,2))
nice <- (actual > 0.005)
m <- max(actual)
scatter_comparison(x = actual[nice], main = "EIDPM", y = eidpm_est[nice], ylab = 'Estimates (EIDPM)', xlab = 'Population')
#legend('topleft', pch = c(1,3), legend=c('Uncontaminated', 'Noisy'))

scatter_comparison(x = actual[nice], main = "FH", y = fh_est[nice], ylab = 'Estimates (FH)', xlab = 'Population')
